@@ -4,9 +4,10 @@ use globset::{GlobSet, GlobSetBuilder};
 use itertools::Itertools;
 use rustc_hash::FxHashSet;
 use sha2::{Digest, Sha256};
-use std::io::{BufReader, Read, Write};
+use std::io::{BufReader, Read, Seek, Write};
 use std::path::{Component, Path, PathBuf};
 use std::{io, mem};
+use tempfile::NamedTempFile;
 use tracing::{debug, trace};
 use walkdir::WalkDir;
 use zip::{CompressionMethod, ZipWriter};
@@ -50,7 +51,13 @@ pub fn build_wheel(
 
     let wheel_path = wheel_dir.join(filename.to_string());
     debug!("Writing wheel at {}", wheel_path.user_display());
-    let wheel_writer = ZipDirectoryWriter::new_wheel(File::create(&wheel_path)?);
+
+    if wheel_path.exists() {
+        fs_err::remove_file(&wheel_path)?;
+    }
+
+    let temp_file = NamedTempFile::new_in(wheel_dir)?;
+    let wheel_writer = ZipDirectoryWriter::new_wheel(temp_file.as_file());
 
     write_wheel(
         source_tree,
@@ -61,6 +68,10 @@ pub fn build_wheel(
         show_warnings,
     )?;
 
+    temp_file
+        .persist(&wheel_path)
+        .map_err(|err| Error::Persist(wheel_path.clone(), err.error))?;
+
     Ok(filename)
 }
 
@@ -295,7 +306,13 @@ pub fn build_editable(
 
     let wheel_path = wheel_dir.join(filename.to_string());
     debug!("Writing wheel at {}", wheel_path.user_display());
-    let mut wheel_writer = ZipDirectoryWriter::new_wheel(File::create(&wheel_path)?);
+
+    if wheel_path.exists() {
+        fs_err::remove_file(&wheel_path)?;
+    }
+
+    let temp_file = NamedTempFile::new_in(wheel_dir)?;
+    let mut wheel_writer = ZipDirectoryWriter::new_wheel(temp_file.as_file());
 
     debug!("Adding pth file to {}", wheel_path.user_display());
     // Check that a module root exists in the directory we're linking from the `.pth` file
@@ -323,6 +340,10 @@ pub fn build_editable(
     )?;
     wheel_writer.close(&dist_info_dir)?;
 
+    temp_file
+        .persist(&wheel_path)
+        .map_err(|err| Error::Persist(wheel_path.clone(), err.error))?;
+
     Ok(filename)
 }
 
@@ -615,18 +636,18 @@ fn wheel_info(filename: &WheelFilename, uv_version: &str) -> String {
 }
 
 /// Zip archive (wheel) writer.
-struct ZipDirectoryWriter {
-    writer: ZipWriter<File>,
+struct ZipDirectoryWriter<W: Write + Seek> {
+    writer: ZipWriter<W>,
     compression: CompressionMethod,
     /// The entries in the `RECORD` file.
     record: Vec<RecordEntry>,
 }
 
-impl ZipDirectoryWriter {
+impl<W: Write + Seek> ZipDirectoryWriter<W> {
     /// A wheel writer with deflate compression.
-    fn new_wheel(file: File) -> Self {
+    fn new_wheel(writer: W) -> Self {
         Self {
-            writer: ZipWriter::new(file),
+            writer: ZipWriter::new(writer),
             compression: CompressionMethod::Deflated,
             record: Vec::new(),
         }
@@ -636,9 +657,9 @@ impl ZipDirectoryWriter {
     ///
     /// Since editables are temporary, we save time be skipping compression and decompression.
     #[expect(dead_code)]
-    fn new_editable(file: File) -> Self {
+    fn new_editable(writer: W) -> Self {
         Self {
-            writer: ZipWriter::new(file),
+            writer: ZipWriter::new(writer),
             compression: CompressionMethod::Stored,
             record: Vec::new(),
         }
@@ -660,7 +681,7 @@ impl ZipDirectoryWriter {
     }
 }
 
-impl DirectoryWriter for ZipDirectoryWriter {
+impl<W: Write + Seek> DirectoryWriter for ZipDirectoryWriter<W> {
     fn write_bytes(&mut self, path: &str, bytes: &[u8]) -> Result<(), Error> {
         trace!("Adding {}", path);
         // Set appropriate permissions for metadata files (644 = rw-r--r--)
@@ -840,7 +861,7 @@ mod test {
     #[test]
     fn test_prepare_metadata() {
         let metadata_dir = TempDir::new().unwrap();
-        let built_by_uv = Path::new("../../scripts/packages/built-by-uv");
+        let built_by_uv = Path::new("../../test/packages/built-by-uv");
         metadata(built_by_uv, metadata_dir.path(), "1.0.0+test").unwrap();
 
         let mut files: Vec<_> = WalkDir::new(metadata_dir.path())
