@@ -26,28 +26,29 @@ use std::str::FromStr;
 use serde::{Deserialize, Deserializer, Serialize, Serializer, de};
 use thiserror::Error;
 use url::Url;
-use uv_cache_key::{CacheKey, CacheKeyHasher};
 
-use cursor::Cursor;
-pub use marker::{
+use uv_cache_key::{CacheKey, CacheKeyHasher};
+use uv_normalize::{ExtraName, PackageName};
+
+use crate::cursor::Cursor;
+pub use crate::marker::{
     CanonicalMarkerValueExtra, CanonicalMarkerValueString, CanonicalMarkerValueVersion,
     ContainsMarkerTree, ExtraMarkerTree, ExtraOperator, InMarkerTree, MarkerEnvironment,
     MarkerEnvironmentBuilder, MarkerExpression, MarkerOperator, MarkerTree, MarkerTreeContents,
     MarkerTreeKind, MarkerValue, MarkerValueExtra, MarkerValueList, MarkerValueString,
     MarkerValueVersion, MarkerWarningKind, StringMarkerTree, StringVersion, VersionMarkerTree,
 };
-pub use origin::RequirementOrigin;
+pub use crate::origin::RequirementOrigin;
 #[cfg(feature = "non-pep508-extensions")]
-pub use unnamed::{UnnamedRequirement, UnnamedRequirementUrl};
-pub use uv_normalize::{ExtraName, InvalidNameError, PackageName};
+pub use crate::unnamed::{UnnamedRequirement, UnnamedRequirementUrl};
+pub use crate::verbatim_url::{
+    Scheme, VerbatimUrl, VerbatimUrlError, expand_env_vars, looks_like_git_repository,
+    split_scheme, strip_host,
+};
 /// Version and version specifiers used in requirements (reexport).
 // https://github.com/konstin/pep508_rs/issues/19
 pub use uv_pep440;
 use uv_pep440::{VersionSpecifier, VersionSpecifiers};
-pub use verbatim_url::{
-    Scheme, VerbatimUrl, VerbatimUrlError, expand_env_vars, looks_like_git_repository,
-    split_scheme, strip_host,
-};
 
 mod cursor;
 pub mod marker;
@@ -1044,6 +1045,41 @@ fn parse_pep508_requirement<T: Pep508Url>(
     })
 }
 
+#[cfg(feature = "rkyv")]
+/// An [`rkyv`] implementation for [`Requirement`].
+impl<T: Pep508Url + Display> rkyv::Archive for Requirement<T> {
+    type Archived = rkyv::string::ArchivedString;
+    type Resolver = rkyv::string::StringResolver;
+
+    #[inline]
+    fn resolve(&self, resolver: Self::Resolver, out: rkyv::Place<Self::Archived>) {
+        let as_str = self.to_string();
+        rkyv::string::ArchivedString::resolve_from_str(&as_str, resolver, out);
+    }
+}
+
+#[cfg(feature = "rkyv")]
+impl<T: Pep508Url + Display, S> rkyv::Serialize<S> for Requirement<T>
+where
+    S: rkyv::rancor::Fallible + rkyv::ser::Allocator + rkyv::ser::Writer + ?Sized,
+    S::Error: rkyv::rancor::Source,
+{
+    fn serialize(&self, serializer: &mut S) -> Result<Self::Resolver, S::Error> {
+        let as_str = self.to_string();
+        rkyv::string::ArchivedString::serialize_from_str(&as_str, serializer)
+    }
+}
+
+#[cfg(feature = "rkyv")]
+impl<T: Pep508Url + Display, D: rkyv::rancor::Fallible + ?Sized>
+    rkyv::Deserialize<Requirement<T>, D> for rkyv::string::ArchivedString
+{
+    fn deserialize(&self, _deserializer: &mut D) -> Result<Requirement<T>, D::Error> {
+        // SAFETY: We only serialize valid requirements.
+        Ok(Requirement::<T>::from_str(self.as_str()).unwrap())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     //! Half of these tests are copied from <https://github.com/pypa/packaging/pull/624>
